# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

from __future__ import absolute_import

import json
import time

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import (
    Temp,
    Fan,
    PSU)
from netq_lib.common.enums import ServiceStatus

local_sensor_dict = dict()


def process_sensors(output, svc_failed=False, **kwargs):
    if svc_failed:
        return ServiceStatus.FAIL
    else:
        try:
            s_list = json.loads(output)
        except ValueError:
            return ServiceStatus.FAIL

    for s in s_list:
        sensor = None
        name = s['name']
        cur_st = s['state'].encode('ascii', 'ignore').lower()
        if s['type'] == 'temp':
            sensor = Temp()
            if 'input' in s:
                sensor.s_input = s['input']
                sensor.s_crit = int(s['crit'])
                sensor.s_max = int(s['max'])
                sensor.s_min = int(s['min'])
                sensor.s_lcrit = int(s['lcrit'])
            else:
                sensor.s_input = 0.0
                sensor.s_max = 0
                sensor.s_min = 0
                sensor.s_crit = 0
                sensor.s_lcrit = 0
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()
            if local_sensor_dict.get(name):
                prev_val, prev_st = local_sensor_dict[name]
                if ((cur_st == prev_st) and
                        (abs(sensor.s_input - prev_val) < 2.5)):
                    continue
            local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'fan':
            sensor = Fan()
            if 'input' in s:
                sensor.s_input = s['input']
                sensor.s_max = s['max']
                sensor.s_min = s['min']
                s_max_val = int(float(sensor.s_max) * 0.1)
            else:
                sensor.s_input = 0
                sensor.s_max = 0
                sensor.s_min = 0
                s_max_val = 300
            sensor.s_desc = s['description'].encode('ascii', 'ignore').lower()
            if local_sensor_dict.get(name):
                prev_val, prev_st = local_sensor_dict[name]

                if ((cur_st == prev_st) and
                        abs(sensor.s_input - prev_val) < s_max_val):
                    continue
            local_sensor_dict[name] = (sensor.s_input, cur_st)
        elif s['type'] == 'power':
            sensor = PSU()

        if sensor is not None:
            sensor.s_name = s['name'].encode('ascii', 'ignore').lower()
            if not s['msg']:
                sensor.s_msg = ''
            else:
                msg = s['msg'].encode('ascii', 'ignore')

                msgs = msg.split('\n')
                m = []
                for msg in msgs:
                    m.append(msg.split(':', 1)[1])
                sensor.s_msg = ','.join(m)
            sensor.s_state = s['state'].encode('ascii', 'ignore').lower()
            sensor.s_prev_state = s['prev_state'].encode('ascii',
                                                         'ignore').lower()
            sensor.save()
    return ServiceStatus.OK
